-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2011.11.06					---
--- Version: v0.3 [2011.11.07]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/fastercamera
--- WoWInterface	http://www.wowinterface.com/downloads/info20483-FasterCamera.html

local VERSION = 0.3
local L = LibStub("AceLocale-3.0"):GetLocale("FasterCamera", true)

	----------------
	--- Prehooks ---
	----------------

local oldZoomIn = CameraZoomIn
local oldZoomOut = CameraZoomOut

function CameraZoomIn(distance)
	oldZoomIn(FasterCameraDB)
end

function CameraZoomOut(distance)
	oldZoomOut(FasterCameraDB)
end

	---------------
	--- Options ---
	---------------

local cvar = {
	"cameraDistanceMoveSpeed",
	"cameraDistanceMax",
	"cameraDistanceMaxFactor",
}

local options = {
	type = "group",
	name = "FasterCamera |cffADFF2Fv"..VERSION.."|r",
	args = {
		inline = {
			type = "group",
			order = 1,
			name = " ",
			inline = true,
			args = {
				speed = {
					type = "range",
					order = 1,
					width = "double",
					descStyle = "",
					name = "|cffFFFFFF"..L["Camera Zoom Increment"].."|r",
					get = function(i) return FasterCameraDB end,
					set = function(i, v) FasterCameraDB = v end,
					min = 1,
					max = 50, softMax = 20,
					step = 1,
				},
				newline1 = {type = "description", order = 2, name = "\n"},
				cameraDistanceMoveSpeed = {
					type = "range",
					order = 3,
					width = "double",
					descStyle = "",
					name = "CVar |cffFFFF00cameraDistanceMoveSpeed|r",
					get = function(i) return tonumber(GetCVar("cameraDistanceMoveSpeed")) end,
					set = function(i, v) SetCVar("cameraDistanceMoveSpeed", v) end,
					min = 1,
					max = 50,
					step = 1,
				},
				newline2 = {type = "description", order = 4, name = "\n"},
				cameraDistanceMax = {
					type = "range",
					order = 5,
					width = "double",
					descStyle = "",
					name = "CVar |cff71D5FFcameraDistanceMax|r",
					get = function(i) return tonumber(GetCVar("cameraDistanceMax")) end,
					set = function(i, v) SetCVar("cameraDistanceMax", v) end,
					min = 1,
					max = 50,
					step = 1,
				},
				newline3 = {type = "description", order = 6, name = "\n"},
				cameraDistanceMaxFactor = {
					type = "range",
					order = 7,
					width = "double",
					descStyle = "",
					name = "CVar |cff71D5FFcameraDistanceMaxFactor|r",
					get = function(i) return tonumber(GetCVar("cameraDistanceMaxFactor")) end,
					set = function(i, v) SetCVar("cameraDistanceMaxFactor", v) end,
					min = 1,
					max = 50, softMax = 4,
					step = 0.5,
				},
			},
		},
		feedback = {
			type = "description",
			order = 2,
			fontSize = "large",
			name = function()
				local cameraDistanceMax = GetCVar("cameraDistanceMax")
				local cameraDistanceMaxFactor = GetCVar("cameraDistanceMaxFactor")
				local maxDistance = cameraDistanceMax * cameraDistanceMaxFactor
				local maxString = maxDistance<=50 and "|cff71D5FF"..maxDistance.."|r" or maxDistance.." |cff71D5FF(50)|r"
				return format("\n"..L["Max Camera Distance"].." = |cff71D5FF%s|r * |cff71D5FF%s|r = %s", cameraDistanceMax, cameraDistanceMaxFactor, maxString)
			end,
		},
		spacing1 = {type = "description", order = 3, name = "\n\n\n\n\n\n\n\n\n\n\n\n\n\n"},
		reset = {
			type = "execute",
			order = 4,
			descStyle = "",
			name = RESET,
			confirm = true, confirmText = RESET.."?",
			func = function()
				FasterCameraDB = 5
				for _, v in ipairs(cvar) do
					SetCVar(v, GetCVarDefault(v))
				end
			end,
		},
	},
}

	----------------------
	--- Initialization ---
	----------------------

local optionsFrame

local function OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and ... == "FasterCamera" then
		self:UnregisterEvent("ADDON_LOADED")
		FasterCameraDB = FasterCameraDB or 5
		LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("FasterCamera", options)
		optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FasterCamera", "FasterCamera")
	end
end

local f = CreateFrame("Frame")
f:RegisterEvent("ADDON_LOADED")
f:SetScript("OnEvent", OnEvent)

	---------------------
	--- Slash Command ---
	---------------------

SLASH_FASTERCAMERA1 = "/fc"
SLASH_FASTERCAMERA2 = "/fastercam"
SLASH_FASTERCAMERA3 = "/fastercamera"

SlashCmdList["FASTERCAMERA"] = function(msg, editbox)
	InterfaceOptionsFrame_OpenToCategory(optionsFrame)
end

	---------------------
	--- LibDataBroker ---
	---------------------
	
local function maxString()
	local maxDistance = GetCVar("cameraDistanceMax") * GetCVar("cameraDistanceMaxFactor")
	return maxDistance<=50 and maxDistance or 50
end

local dataobject = {
	type = "launcher",
	icon = "Interface\\Icons\\Spell_ChargePositive",
	text = "FasterCamera",
	OnClick = function(clickedframe, button)
		if InterfaceOptionsFrame:IsShown() and strfind(InterfaceOptionsFramePanelContainer.displayedPanel.name, "FasterCamera") then
			InterfaceOptionsFrame:Hide()
		else
			InterfaceOptionsFrame_OpenToCategory(optionsFrame)
		end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine("|cffADFF2FFasterCamera|r")
		tt:AddDoubleLine(L["Zoom Increment"], "|cffFFFFFFx"..FasterCameraDB.."|r")
		tt:AddDoubleLine(L["Max Distance"], "|cffFFFFFF"..maxString().."|r")
	end,
}

LibStub("LibDataBroker-1.1"):NewDataObject("FasterCamera", dataobject)